<?php
/*
Plugin Name: Lockdown WordPress Admin
Plugin URI: http://talkingwithsean.com/2011/01/lockdown-wp-admin/
Description: Securing the WordPress Administration interface.
Version: 1.3.2
Author: Sean Fisher
Author URI: http://talkingwithsean.com/
License: GPL v3
*/

//	This file name
define('LD_FILE_NAME', __FILE__ );

/**
 *	This is the plugin that will add security to our site
 *
 *	@author Sean Fisher <me@tlksean.me>
 *	@version 1.1.2
 *	@license GPL v3
**/
class WP_LockAuth {
	
	/**
	 * The HTTP Auth name for the protected area
	 * Change this via calling the object, not by editing the file.
	 *
	 * @access public
	 * @global string
	**/
	public $relm = "Secure Area";
	
	/**
	 * The current user ID from our internal array
	 *
	 * @access private
	**/
	private $current_user = FALSE;
	
	/**
	 * The base to get the login url
	 *
	 * @access private
	**/
	private $login_base = FALSE;
	
	function WP_LockAuth()
	{
		//	We don't like adding network wide WordPress plugins.
		require_once( dirname( __FILE__ ) .'/no-wpmu.php' );
		
		//	Add the action to setup the menu.
		add_action('admin_menu', array( &$this, 'add_admin_menu'));
		
		//	Setup the plugin.
		$this->setup_hide_admin();
		
		//	Hide the login form
		$this->redo_login_form();
		
		//	We no longer update the options here, but rather when we call on the callback function from the menu.
		//	More secure.
	}
	
	/**
	 * Update the users
	 *
	 * @access private
	**/
	function update_users()
	{
		if ( !isset( $_GET['page'] ) )
			return;
		
		if ( $_GET['page'] !== 'lockdown-private-users' )
			return;
		
		//	Nonce
		if ( !isset( $_POST['_wpnonce'] ) )
			return;
		
		$nonce = $_POST['_wpnonce'];
		if (! wp_verify_nonce($nonce, 'lockdown-wp-admin') )
			wp_die('Security error, please try again.');
		
		//	---------------------------------------------------
		
		//	Add a user
		if ( isset( $_POST['private_username'] ) && isset( $_POST['private_password'] ) )
		{
			if ( $_POST['private_username'] !== '' && $_POST['private_password'] !== '' )
			{
				//	Adding a user.
				$users = $this->get_private_users();
				$add['user'] = sanitize_user( $_POST['private_username'] );
				$add['pass'] = trim( md5( $_POST['private_password'] ) );
				
				$users[] = $add;
				
				update_option('ld_private_users', $users);
				
				define('LD_WP_ADMIN', TRUE);
				//wp_redirect( admin_url('admin.php?page=lockdown-private-users&updated=true'));
				return;
			}
		}
		
		//	Deleting a user.
		if ( isset( $_GET['delete'] ) && isset( $_GET['hash'] ) )
		{
			//	Check hash
			if ( $_GET['hash'] !== wp_hash(get_current_user_id()) )
				return;
			
			//	Delete the user.
			unset( $users );
			$users = $this->get_private_users();
			$to_delete = (int) $_GET['delete'];
			
			if ( count( $users ) > 0 )
			{
				foreach( $users as $key => $val )
				{
					if ( $key === $to_delete ) :
						if( $this->current_user !== '' && $to_delete === $this->current_user )
						{
							//	They can't delete themselves!
							define('LD_ERROR', 'delete-self');
							return;
						}
						
						unset( $users[$key] );
					endif;
				}			
			}
			
			update_option('ld_private_users', $users);
			
			define('LD_WP_ADMIN', TRUE);
			return;
		}
	}
	
	/**
	 * Update the options
	 *
	 * @access private
	**/
	function update_options()
	{
		if ( !isset( $_GET['page'] ) )
			return;
		
		if ( $_GET['page'] !== 'lockdown-wp-admin' )
			return;
		
		if ( !isset( $_POST['did_update'] ) )
			return;
		
		//	Nonce
		$nonce = $_POST['_wpnonce'];
		if (! wp_verify_nonce($nonce, 'lockdown-wp-admin') )
			wp_die('Security error, please try again.');
		
		//	---------------------------------------------------
		//	They're updating.
		if ( isset( $_POST['http_auth'] ) )
			update_option('ld_http_auth', trim( strtolower( $_POST['http_auth'] ) ) );
		else
			update_option('ld_http_auth', 'none' );
		
		if ( !isset( $_POST['hide_wp_admin'] ) )
		{
			update_option('ld_hide_wp_admin', 'nope');
		}
		else
		{
			if ( $_POST['hide_wp_admin'] === 'yep' )
				update_option('ld_hide_wp_admin', 'yep');
			else
				update_option('ld_hide_wp_admin', 'nope');
		}
		
		if ( isset( $_POST['login_base'] ) )
		{
			$base = sanitize_title_with_dashes( $_POST['login_base']);
			$base = str_replace('/', '', $base);
			
			$disallowed = array(
				'user', 'wp-admin', 'wp-content', 'wp-includes', 'wp-feed.php', 'index', 'feed', 'rss', 'robots', 'robots.txt'
			);
			if ( in_array( $base, $disallowed ) )
			{
				define('LD_DIS_BASE', TRUE);
			}
			else
			{
				
				update_option('ld_login_base', $base);
				$this->login_base = sanitize_title_with_dashes ( $base );
			}
		}
		
		//	Redirect
		define('LD_WP_ADMIN', TRUE);
		return;
	}
	
	/**
	 * Send headers to the browser that are going to ask for a username/pass
	 * from the browser.
	 *
	 * @access private
	 * @return void
	**/
	private function inauth_headers()
	{
		header('WWW-Authenticate: Basic realm="'.$this->relm.'"');
		header('HTTP/1.0 401 Unauthorized');
		echo '<h1>Authorization Required.</h1>';
		exit;
	}
	
	/**
	 * Check for a HTTP auth session
	 *
	 * If they find one, we will setup the 'INTERNAL_AUTH_PASSED' constant.
	 * If they failed, it will send the HTTP auth headers to get the username/
	 * password.
	 *
	 * @uses self::inauth_headers() When we need the username/pass
	 * @access public
	**/
	public function setup()
	{
		/* Check for values in $PHP_AUTH_USER and $PHP_AUTH_PW */
		if ((!isset($_SERVER['PHP_AUTH_USER'])) || (!isset($_SERVER['PHP_AUTH_PW']))) {
			$this->inauth_headers();
		
		} else if ((isset($_SERVER['PHP_AUTH_USER'])) && (isset($_SERVER['PHP_AUTH_PW']))){
			
			/* Values contain some values, so check to see if they're correct */
			
			if (($_SERVER['PHP_AUTH_USER'] != $this->current_user) || (md5($_SERVER['PHP_AUTH_PW']) != $this->current_pass)) {
				 /* If either the username entered is incorrect, or the password entered is incorrect, send the headers causing dialog box to appear */
				 $this->inauth_headers();
				 
			} else if (($_SERVER['PHP_AUTH_USER'] === $this->current_user) || ( md5($_SERVER['PHP_AUTH_PW'] ) === $this->current_pass)) {
				
				 /* if both values are correct, print success message */
				 //	We're good here!
				 define('INTERNAL_AUTH_PASSED', TRUE);
			}
		} 
	}
	
	/**
	 * Get the users for the private creds
	 *
	 * @access private
	**/
	function get_private_users()
	{
		$opt = get_option('ld_private_users');
		if ( !is_array( $opt ) )
			return array();
		
		return $opt;
	}
	
	/**
	 * Setup hiding wp-admin
	 *
	 * @access void
	**/
	function setup_hide_admin()
	{
		$opt = get_option('ld_hide_wp_admin');
		
		//	Nope!
		if ( $opt !== 'yep' )
			return;
		
		//	We're gonna hide it.
		$no_check_files = array('async-upload.php');
		$no_check_files = apply_filters('no_check_files', $no_check_files);
		
		$explode = explode('/', $_SERVER['SCRIPT_FILENAME'] );
		$file = end( $explode );
    	if ( in_array( $file, $no_check_files ) )
    	{
			define('INTERNAL_AUTH_PASSED', TRUE);
			return;
		}
    	
		//	We only will hide it if we are in admin (/wp-admin/)
		if ( is_admin() )
		{
			//	Non logged in users.
			if ( !is_user_logged_in() )
			{
				//	If they AREN'T logged in and they tried to access wp-admin
				//	we'll just serve them a 404!
				status_header(404);
				require( get_404_template() );
				
				exit;
			}
			
			//	Setup HTTP auth.
			$this->setup_http_area();
		}
	}
	
	/**
	 * Get the current file name
	 *
	 * @return string JUST the file name
	**/
	function get_file()
	{
		//	We're gonna hide it.
		$no_check_files = array('async-upload.php');
		$no_check_files = apply_filters('no_check_files', $no_check_files);
		
		$explode = explode('/', $_SERVER['SCRIPT_FILENAME'] );
		return end( $explode );
	}
	
	/**
	 * Setting up the HTTP Auth
	 *
	 * Here, we only check if it's enabled
	 *
	 * @access private
	**/
	function setup_http_area()
	{
		//	We save what type of auth we're doing here.
		$opt = get_option('ld_http_auth');
		
		switch( $opt )
		{
			//	HTTP auth is going to ask for their WordPress creds.
			case('wp_creds');
				
				/* Check for values in $PHP_AUTH_USER and $PHP_AUTH_PW */
				if (!isset($_SERVER['PHP_AUTH_USER']) || !isset($_SERVER['PHP_AUTH_PW']))
					$this->inauth_headers();
				
				//	Are they already logged in as this?
				$current_uid = get_current_user_id();
				
				//	We fixed this for use with non WP-MS sites
				$requested_user = get_user_by('login', $_SERVER['PHP_AUTH_USER']);
				
				//	Not a valid user.
				if ( !$requested_user )
					$this->inauth_headers();
				
				//	The correct User ID.
				$requested_uid = (int) $requested_user->ID;
				
				//	Already logged in?
				if ( $current_uid === $requested_uid )
				{
					define('INTERNAL_AUTH_PASSED', TRUE);
					return;
				}
				
				//	Attempt to sign them in if they aren't alerady
				if ( !is_user_logged_in() ) :
					//	Try it via wp_signon
					$creds = array();
					$creds['user_login'] = $_SERVER['PHP_AUTH_USER'];
					$creds['user_password'] = $_SERVER['PHP_AUTH_PW'];
					$creds['remember'] = true;
					$user = wp_signon( $creds, false );
					
					//	In error :(
					if ( is_wp_error($user) )
						$this->inauth_headers();
				endif;
				
				//	They passed!
				define('INTERNAL_AUTH_PASSED', TRUE);
				break;
			
			case('private');
				$users = $this->get_private_users();
				
				//	We want a user to exist
				//	If nobody is found, we won't lock them out!
				if ( !$users || !is_array( $users ) )
					return;
				
				//	Let's NOT lock everybody out
				if ( count( $users ) === 0 )
					return;
				
				/* Check for values in $PHP_AUTH_USER and $PHP_AUTH_PW */
				if ( !isset( $_SERVER['PHP_AUTH_USER'] ) || !isset( $_SERVER['PHP_AUTH_PW'] ) )
					$this->inauth_headers();
				
				//	Did they enter a valid user?
				if ( $this->user_array_check( $users, $_SERVER['PHP_AUTH_USER'], $_SERVER['PHP_AUTH_PW'] ) )
				{
					//	Yes!!
					define('INTERNAL_AUTH_PASSED', TRUE);
					$this->set_current_user( $users, $_SERVER['PHP_AUTH_USER'] );
					return;
				}
				else
				{
					//	Nope
					$this->inauth_headers();
					return;
				}
				
				break;
		}
		
	}
	/**
	 * Check an internal array of users against a passed user and pass
	 *
	 * @access public
	 * @return bool
	 *
	 * @param array $array The array of users
	 * @param string $user The username to check for
	 * @param string $pass The password to check for (plain text)
	**/
	function user_array_check( $array, $user, $pass )
	{
		foreach( $array as $key => $val )
		{
			if ( $val['user'] === $user && md5( $pass ) === $val['pass'] )
				return TRUE;
		}
		
		return FALSE;
	}
	
	/**
	 * Set the current user
	 *
	 * @access private
	**/
	function set_current_user( $array, $user )
	{
		foreach( $array as $key => $val )
		{
			if ( $val['user'] === $user )
				$this->current_user = $key;
		}
	}
	
	/**
	 * Adds the admin menu
	 *
	 * @acces private
	**/
	function add_admin_menu()
	{
		add_menu_page('Lockdown WP', 'Lockdown WP', 'manage_options', 'lockdown-wp-admin', array( &$this, 'admin_callback'));
		add_submenu_page( 'lockdown-wp-admin', 'Private Users', 'Private Users', 'manage_options', 'lockdown-private-users',  array( &$this, 'sub_admin_callback'));
	}
	
	/**
	 * The callback for the admin area
	 *
	 * You need the 'manage_options' capability to get here.
	**/
	function admin_callback()
	{
		//	Update the options
		$this->update_options();
		
		//	The UI
		require_once( dirname( __FILE__ ) . '/admin.php' );
	}	
	
	/**
	 * The callback for ther private users management.
	 *
	 * You need the 'manage_options' capability to get here.
	**/
	function sub_admin_callback()
	{
		//	Update the users options
		$this->update_users();
		
		//	The UI
		$private_users = $this->get_private_users();
		require_once( dirname( __FILE__ ) . '/admin-private-users.php' );
	}
	
	/**
	 * Rename the login URL
	 *
	 * @access public
	**/
	public function redo_login_form()
	{
		$login_base = get_option('ld_login_base');
		
		//	It's not enabled.
		if ( $login_base == NULL || !$login_base || $login_base == '' )
			return;
		
		$this->login_base = $login_base;
		unset( $login_base );
		
		//	Setup the filters for the new login form
		add_filter('wp_redirect', array( &$this, 'filter_wp_login'));
		add_filter('network_site_url', array( &$this, 'filter_wp_login'));
		add_filter('site_url', array( &$this, 'filter_wp_login'));
		
		//	The request URL
		$url = str_replace('/', '', $_SERVER['REQUEST_URI'] );;
		
		list( $base, $query ) = explode( '?', $url, 2 );
		
		//	Are they visiting wp-login.php?
		if ( $base == 'wp-login.php')
		{
			status_header(404);
			require( get_404_template() );
			
			exit;
		}
		
		//	Is this the "login" url?
		if ( $base !== $this->login_base )
			return FALSE;
		
		include ABSPATH . "/wp-login.php";
		exit;
	}
	
	/**
	 * Filters out wp-login to whatever they named it
	 *
	 * @access public
	**/
	public function filter_wp_login( $str )
	{
		return str_replace('wp-login.php', $this->login_base, $str);
	}
}

/**
 * The function called at 'init'.
 *
 * Sets up the object
 *
 * @return void
 * @access private
 * @version 1.0
 * @see do_action() Called by the 'init' hook'
**/
function ld_setup_auth()
{
	//	Setup the object.
	$auth_obj = new WP_LockAuth();
}

add_action('init', 'ld_setup_auth');